@@ -1,16 +1,16 @@
-import unittest
-from abc import ABCMeta, abstractmethod
-
-from hdmf import Container
+from hdmf.utils import docval, getargs
+from hdmf import Container, Data
 from hdmf.backends.hdf5 import H5DataIO
 from hdmf.build import (GroupBuilder, DatasetBuilder, ObjectMapper, BuildManager, TypeMap, LinkBuilder,
                         ReferenceBuilder, MissingRequiredBuildWarning, OrphanContainerBuildError,
                         ContainerConfigurationError)
-from hdmf.spec import GroupSpec, AttributeSpec, DatasetSpec, SpecCatalog, SpecNamespace, NamespaceCatalog, RefSpec
+from hdmf.spec import (GroupSpec, AttributeSpec, DatasetSpec, SpecCatalog, SpecNamespace, NamespaceCatalog, RefSpec,
+                       LinkSpec)
 from hdmf.testing import TestCase
-from hdmf.utils import docval, getargs
+from abc import ABCMeta, abstractmethod
+import unittest
 
-from tests.unit.utils import CORE_NAMESPACE
+from tests.unit.utils import CORE_NAMESPACE, create_test_type_map
 
 
 class Bar(Container):
@@ -69,6 +69,10 @@ class Bar(Container):
             self._remove_child(self.__foo)
 
 
+class SubBar(Bar):
+    pass
+
+
 class Foo(Container):
 
     @property
@@ -76,37 +80,189 @@ class Foo(Container):
         return 'Foo'
 
 
+class FooData(Data):
+
+    @property
+    def data_type(self):
+        return 'FooData'
+
+
 class TestGetSubSpec(TestCase):
 
     def setUp(self):
-        self.bar_spec = GroupSpec('A test group specification with a data type', data_type_def='Bar')
-        spec_catalog = SpecCatalog()
-        spec_catalog.register_spec(self.bar_spec, 'test.yaml')
-        namespace = SpecNamespace('a test namespace', CORE_NAMESPACE, [{'source': 'test.yaml'}],
-                                  version='0.1.0',
-                                  catalog=spec_catalog)
-        namespace_catalog = NamespaceCatalog()
-        namespace_catalog.add_namespace(CORE_NAMESPACE, namespace)
-        self.type_map = TypeMap(namespace_catalog)
-        self.type_map.register_container_type(CORE_NAMESPACE, 'Bar', Bar)
+        self.bar_spec = GroupSpec(doc='A test group specification with a data type', data_type_def='Bar')
+        self.sub_bar_spec = GroupSpec(
+            doc='A test group specification with a data type',
+            data_type_def='SubBar',
+            data_type_inc='Bar'
+        )
+        self.type_map = create_test_type_map([self.bar_spec, self.sub_bar_spec], {'Bar': Bar, 'SubBar': SubBar})
 
-    def test_get_subspec_data_type_noname(self):
-        parent_spec = GroupSpec('Something to hold a Bar', 'bar_bucket', groups=[self.bar_spec])
-        sub_builder = GroupBuilder('my_bar', attributes={'data_type': 'Bar', 'namespace': CORE_NAMESPACE,
-                                                         'object_id': -1})
-        GroupBuilder('bar_bucket', groups={'my_bar': sub_builder})
+    def test_bad_name(self):
+        """Test get_subspec on a builder that doesn't map to the spec."""
+        parent_spec = GroupSpec(doc='Empty group', name='bar_bucket')
+        sub_builder = GroupBuilder(
+            name='my_bar',
+            attributes={
+                'data_type': 'Bar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        GroupBuilder(name='bar_bucket', groups={'my_bar': sub_builder})  # add sub_builder as a child to bar_bucket
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIsNone(result)
+
+    def test_bad_name_no_data_type(self):
+        """Test get_subspec on a builder without a data type that doesn't map to the spec."""
+        parent_spec = GroupSpec(doc='Empty group', name='bar_bucket')
+        sub_builder = GroupBuilder(name='my_bar')
+        GroupBuilder(name='bar_bucket', groups={'my_bar': sub_builder})  # add sub_builder as a child to bar_bucket
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIsNone(result)
+
+    def test_unnamed_group_data_type_def(self):
+        """Test get_subspec on a builder that maps to an unnamed subgroup of the given spec using data_type_def."""
+        parent_spec = GroupSpec(
+            doc='Something to hold a Bar',
+            name='bar_bucket',
+            groups=[self.bar_spec]  # using data_type_def, no name
+        )
+        sub_builder = GroupBuilder(
+            name='my_bar',
+            attributes={
+                'data_type': 'Bar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        GroupBuilder(name='bar_bucket', groups={'my_bar': sub_builder})  # add sub_builder as a child to bar_bucket
         result = self.type_map.get_subspec(parent_spec, sub_builder)
         self.assertIs(result, self.bar_spec)
 
-    def test_get_subspec_named(self):
-        child_spec = GroupSpec('A test group specification with a data type', 'my_subgroup')
-        parent_spec = GroupSpec('Something to hold a Bar', 'my_group', groups=[child_spec])
-        sub_builder = GroupBuilder('my_subgroup', attributes={'data_type': 'Bar', 'namespace': CORE_NAMESPACE,
-                                                              'object_id': -1})
-        GroupBuilder('my_group', groups={'my_bar': sub_builder})
+    def test_unnamed_group_data_type_inc(self):
+        """Test get_subspec on a builder that maps to an unnamed subgroup of the given spec using data_type_inc."""
+        inc_spec = GroupSpec(
+            doc='This Bar',
+            data_type_inc='Bar'
+        )
+        parent_spec = GroupSpec(
+            doc='Something to hold a Bar',
+            name='bar_bucket',
+            groups=[inc_spec]  # using data_type_inc
+        )
+        sub_builder = GroupBuilder(
+            name='my_bar',
+            attributes={
+                'data_type': 'Bar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        GroupBuilder(name='bar_bucket', groups={'my_bar': sub_builder})  # add sub_builder as a child to bar_bucket
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIs(result, inc_spec)
+
+    def test_named_group(self):
+        """Test get_subspec on a builder that maps to a named subgroup of the given spec."""
+        # NOTE this works despite the fact that child_spec has no data type but the builder has a data type because
+        # get_subspec acts on the name and not necessarily the data type
+        child_spec = GroupSpec(doc='A test group specification', name='my_subgroup')
+        parent_spec = GroupSpec(doc='Something to hold a Bar', name='my_group', groups=[child_spec])
+        sub_builder = GroupBuilder(
+            name='my_subgroup',
+            attributes={
+                'data_type': 'Bar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        GroupBuilder(name='my_group', groups={'my_bar': sub_builder})  # add sub_builder as a child to my_group
         result = self.type_map.get_subspec(parent_spec, sub_builder)
         self.assertIs(result, child_spec)
 
+    def test_named_dataset(self):
+        """Test get_subspec on a builder that maps to a named dataset of the given spec."""
+        # NOTE this works despite the fact that child_spec has no data type but the builder has a data type because
+        # get_subspec acts on the name and not necessarily the data type
+        child_spec = DatasetSpec(doc='A test dataset specification', name='my_dataset')
+        parent_spec = GroupSpec(doc='Something to hold a Bar', name='my_group', datasets=[child_spec])
+        sub_builder = DatasetBuilder(
+            name='my_dataset',
+            data=[],
+            attributes={
+                'data_type': 'FooData',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        GroupBuilder(name='my_group', datasets={'my_dataset': sub_builder})  # add sub_builder as a child to my_group
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIs(result, child_spec)
+
+    def test_unnamed_link_data_type_inc(self):
+        """Test get_subspec on a builder that maps to an unnamed link."""
+        link_spec = LinkSpec(doc='This Bar', target_type='Bar')
+        parent_spec = GroupSpec(
+            doc='Something to hold a Bar',
+            name='bar_bucket',
+            links=[link_spec]
+        )
+        bar_builder = GroupBuilder(
+            name='my_bar',
+            attributes={
+                'data_type': 'Bar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        sub_builder = LinkBuilder(builder=bar_builder, name='my_link')
+        GroupBuilder(name='bar_bucket', links={'my_bar': sub_builder})
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIs(result, link_spec)
+
+    def test_named_link_data_type_inc(self):
+        """Test get_subspec on a builder that maps to an named link."""
+        link_spec = LinkSpec(doc='This Bar', target_type='Bar', name='bar_link')
+        parent_spec = GroupSpec(
+            doc='Something to hold a Bar',
+            name='bar_bucket',
+            links=[link_spec]
+        )
+        bar_builder = GroupBuilder(
+            name='my_bar',
+            attributes={
+                'data_type': 'Bar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        sub_builder = LinkBuilder(builder=bar_builder, name='bar_link')
+        GroupBuilder(name='bar_bucket', links={'my_bar': sub_builder})
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIs(result, link_spec)
+
+    def test_named_link_hierarchy_data_type_inc(self):
+        """Test get_subspec on a builder that maps to an named link."""
+        link_spec = LinkSpec(doc='This Bar', target_type='Bar', name='bar_link')
+        parent_spec = GroupSpec(
+            doc='Something to hold a Bar',
+            name='bar_bucket',
+            links=[link_spec]
+        )
+        bar_builder = GroupBuilder(
+            name='my_bar',
+            attributes={
+                'data_type': 'SubBar',
+                'namespace': CORE_NAMESPACE,
+                'object_id': -1
+            }
+        )
+        sub_builder = LinkBuilder(builder=bar_builder, name='bar_link')
+        GroupBuilder(name='bar_bucket', links={'my_bar': sub_builder})
+        result = self.type_map.get_subspec(parent_spec, sub_builder)
+        self.assertIs(result, link_spec)
+
 
 class TestTypeMap(TestCase):
 
