@@ -118,6 +118,7 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
              'object_id': {0: data1.object_id, 1: data1.object_id,
                            2: data2.object_id, 3: data2.object_id, 4: data2.object_id,
                            5: data3.object_id, 6: data3.object_id},
+             'relative_path': {0: '', 1: '', 2: '', 3: '', 4: '', 5: '', 6: ''},
              'field': {0: 'species', 1: 'species', 2: '', 3: '', 4: '', 5: '', 6: ''},
              'keys_idx': {0: 0, 1: 1, 2: 0, 3: 1, 4: 2, 5: 3, 6: 3},
              'key': {0: 'Mus musculus', 1: 'Homo sapiens', 2: 'Mus musculus', 3: 'Homo sapiens',
@@ -145,7 +146,7 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
         # Convert to dataframe with categories and compare against the expected result
         result_df = er.to_dataframe(use_categories=True)
         cols_with_categories = [
-            ('objects', 'objects_idx'), ('objects', 'object_id'), ('objects', 'field'),
+            ('objects', 'objects_idx'), ('objects', 'object_id'), ('objects', 'relative_path'), ('objects', 'field'),
             ('keys', 'keys_idx'), ('keys', 'key'),
             ('resources', 'resources_idx'), ('resources', 'resource'), ('resources', 'resource_uri'),
             ('entities', 'entities_idx'), ('entities', 'entity_id'), ('entities', 'entity_uri')]
@@ -153,6 +154,108 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
         expected_df = pd.DataFrame.from_dict(expected_df_data)
         pd.testing.assert_frame_equal(result_df, expected_df)
 
+    def test_assert_external_resources_equal(self):
+        er_left = ExternalResources('terms')
+        er_left.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        er_right = ExternalResources('terms')
+        er_right.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        self.assertTrue(ExternalResources.assert_external_resources_equal(er_left,
+                                                                          er_right))
+
+    def test_invalid_keys_assert_external_resources_equal(self):
+        er_left = ExternalResources('terms')
+        er_left.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        er_right = ExternalResources('terms')
+        er_right.add_ref(
+            container='invalid', key='invalid',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        with self.assertRaises(AssertionError):
+            ExternalResources.assert_external_resources_equal(er_left,
+                                                              er_right)
+
+    def test_invalid_objects_assert_external_resources_equal(self):
+        er_left = ExternalResources('terms')
+        er_left.add_ref(
+            container='invalid', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        er_right = ExternalResources('terms')
+        er_right.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        with self.assertRaises(AssertionError):
+            ExternalResources.assert_external_resources_equal(er_left,
+                                                              er_right)
+
+    def test_invalid_resources_assert_external_resources_equal(self):
+        er_left = ExternalResources('terms')
+        er_left.add_ref(
+            container='uuid1', key='key1',
+            resource_name='invalid', resource_uri='invalid',
+            entity_id="id11", entity_uri='url11')
+
+        er_right = ExternalResources('terms')
+        er_right.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        with self.assertRaises(AssertionError):
+            ExternalResources.assert_external_resources_equal(er_left,
+                                                              er_right)
+
+    def test_invalid_entity_assert_external_resources_equal(self):
+        er_left = ExternalResources('terms')
+        er_left.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="invalid", entity_uri='invalid')
+
+        er_right = ExternalResources('terms')
+        er_right.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        with self.assertRaises(AssertionError):
+            ExternalResources.assert_external_resources_equal(er_left,
+                                                              er_right)
+
+    def test_invalid_object_keys_assert_external_resources_equal(self):
+        er_left = ExternalResources('terms')
+        er_left.add_ref(
+            container='invalid', key='invalid',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        er_right = ExternalResources('terms')
+        er_right._add_key('key')
+        er_right.add_ref(
+            container='uuid1', key='key1',
+            resource_name='resource11', resource_uri='resource_uri11',
+            entity_id="id11", entity_uri='url11')
+
+        with self.assertRaises(AssertionError):
+            ExternalResources.assert_external_resources_equal(er_left,
+                                                              er_right)
+
     def test_add_ref(self):
         er = ExternalResources(name='terms')
         data = Data(name="species", data=['Homo sapiens', 'Mus musculus'])
@@ -165,6 +268,54 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
         self.assertEqual(er.entities.data, [(0, 0, 'entity_id1', 'entity1')])
         self.assertEqual(er.objects.data, [(data.object_id, '', '')])
 
+    def test_to_tsv_and_from_tsv(self):
+        # write er to file
+        self.container.to_tsv(path=self.export_filename)
+        # read er back from file and compare
+        er_obj = ExternalResources.from_tsv(path=self.export_filename)
+        # Check that the data is correct
+        ExternalResources.assert_external_resources_equal(er_obj, self.container, check_dtype=False)
+
+    def test_to_tsv_and_from_tsv_missing_keyidx(self):
+        # write er to file
+        df = self.container.to_dataframe(use_categories=True)
+        df.at[0, ('keys', 'keys_idx')] = 10  # Change key_ix 0 to 10
+        df.to_csv(self.export_filename, sep='\t')
+        # read er back from file and compare
+        msg = "Missing keys_idx entries [0, 2, 3, 4, 5, 6, 7, 8, 9]"
+        with self.assertRaisesWith(ValueError, msg):
+            _ = ExternalResources.from_tsv(path=self.export_filename)
+
+    def test_to_tsv_and_from_tsv_missing_objectidx(self):
+        # write er to file
+        df = self.container.to_dataframe(use_categories=True)
+        df.at[0, ('objects', 'objects_idx')] = 10  # Change key_ix 0 to 10
+        df.to_csv(self.export_filename, sep='\t')
+        # read er back from file and compare
+        msg = "Missing objects_idx entries [0, 2, 3, 4, 5, 6, 7, 8, 9]"
+        with self.assertRaisesWith(ValueError, msg):
+            _ = ExternalResources.from_tsv(path=self.export_filename)
+
+    def test_to_tsv_and_from_tsv_missing_resourcesidx(self):
+        # write er to file
+        df = self.container.to_dataframe(use_categories=True)
+        df.at[0, ('resources', 'resources_idx')] = 10  # Change key_ix 0 to 10
+        df.to_csv(self.export_filename, sep='\t')
+        # read er back from file and compare
+        msg = "Missing resources_idx entries [0, 2, 3, 4, 5, 6, 7, 8, 9]"
+        with self.assertRaisesWith(ValueError, msg):
+            _ = ExternalResources.from_tsv(path=self.export_filename)
+
+    def test_to_tsv_and_from_tsv_missing_entitiesidx(self):
+        # write er to file
+        df = self.container.to_dataframe(use_categories=True)
+        df.at[0, ('entities', 'entities_idx')] = 10  # Change key_ix 0 to 10
+        df.to_csv(self.export_filename, sep='\t')
+        # read er back from file and compare
+        msg = "Missing entities_idx entries [0, 2, 3, 4, 5, 6, 7, 8, 9]"
+        with self.assertRaisesWith(ValueError, msg):
+            _ = ExternalResources.from_tsv(path=self.export_filename)
+
     def test_add_ref_duplicate_resource(self):
         er = ExternalResources(name='terms')
         er.add_ref(
@@ -328,14 +479,17 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
 
     def test_get_object_resources(self):
         er = ExternalResources(name='terms')
-        data = Data(name='data_name', data=np.array([('Mus musculus', 9, 81.0), ('Homo sapien', 3, 27.0)],
-                    dtype=[('species', 'U14'), ('age', 'i4'), ('weight', 'f4')]))
+        table = DynamicTable(name='test_table', description='test table description')
+        table.add_column(name='test_col', description='test column description')
+        table.add_row(test_col='Mouse')
 
-        er.add_ref(container=data, key='Mus musculus', resource_name='NCBI_Taxonomy',
-                   resource_uri='https://www.ncbi.nlm.nih.gov/taxonomy',
+        er.add_ref(container=table, attribute='test_col', key='Mouse',
+                   resource_name='NCBI_Taxonomy',
+                   resource_uri='https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi',
                    entity_id='NCBI:txid10090',
-                   entity_uri='https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=10090')
-        received = er.get_object_resources(data)
+                   entity_uri='https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=10090',
+                   )
+        received = er.get_object_resources(table['test_col'])
         expected = pd.DataFrame(
             data=[[0, 0, 'NCBI:txid10090', 'https://www.ncbi.nlm.nih.gov/Taxonomy/Browser/wwwtax.cgi?id=10090']],
             columns=['keys_idx', 'resource_idx', 'entity_id', 'entity_uri'])
@@ -366,7 +520,7 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
 
         self.assertEqual(er.objects.data, [('uuid1', '', ''), (data.object_id, '', '')])
 
-    def test_check_object_field_error(self):
+    def test_check_object_field_multi_error(self):
         er = ExternalResources(name='terms')
         data = Data(name="species", data=['Homo sapiens', 'Mus musculus'])
         er._check_object_field(data, '')
@@ -374,6 +528,12 @@ class TestExternalResources(H5RoundTripMixin, TestCase):
         with self.assertRaises(ValueError):
             er._check_object_field(data, '')
 
+    def test_check_object_field_not_in_obj_table(self):
+        er = ExternalResources(name='terms')
+        data = Data(name="species", data=['Homo sapiens', 'Mus musculus'])
+        with self.assertRaises(ValueError):
+            er._check_object_field(container=data, relative_path='', field='', create=False)
+
     def test_add_ref_attribute(self):
         # Test to make sure the attribute object is being used for the id
         # for the external reference.
